; ------------------------------------------------------------------------
; Modulo Controlador de Radio FM AR1000                         10/10/2019
;
; STATUS: FUNCIONA
;
; Este servico controla um radio FM AR1000 usando modulo MSSP do PIC 
; configurado em modo I2C.
;
; ------------------------------------------------------------------------

; ------------------------------------------------------------------------
; DESCRICAO DAS FUNCOES
; ------------------------------------------------------------------------
; FUNCAO........ ar1000Config
; PARAMETROS.... Nao ha
;
; Configura o radio com os valores iniciais contidos na tabela 'DEFAULTS'
;
; ------------------------------------------------------------------------
; FUNCAO........ _AR1000DEF
; PARAMETROS.... Nao ha
;
; Usada pela funcao 'ar1000Config', esta funcao apenas aponta para a tabela
; 'DEFAULTS'. Nao deve ser usada no programa.
;
; ------------------------------------------------------------------------
; FUNCAO........ _AR1000MVC
; PARAMETROS.... Nao ha
;
; Usada pela funcao 'ar1000Config', esta funcao move 2 bytes da tabela
; 'DEFAULT' para o registrador ARREGS. Nao deve ser usada no programa.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000ReadAll
; PARAMETROS.... Nao ha
;
; Le todos os registradores do AR1000 para ARREGS 36 bytes).
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000SetFreq
; PARAMETROS.... ARFREQ(2) = (10 * FreqReal) - 690
;
; Sintoniza o radi na frequencia especificada no registrador ARFREQ(2).
; O valor de ARFREQ(2) deve ser 10 veze a frequencia real da estacao
; subtraida de 690.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000TuneOn
; PARAMETROS.... Nao ha
;
; Liga o bit _R2<TUNE>.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000TuneOff
; PARAMETROS.... Nao ha
;
; Desiga o bit _R2<TUNE>.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000Mute
; PARAMETROS.... Nao ha
;
; Liga o bit _R1<HMUTE> para desativar o som do radio.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000Play
; PARAMETROS.... Nao ha
;
; Desliga o bit _R1<HMUTE> para ativar o som do radio.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000SetVol
; PARAMETROS.... W contem nivel de volume de 1 a 18 
;
; Modifica o volume do radio.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000Seek
; PARAMETROS.... W = 0 para sintonia abaixo
;                W = 1 para sintonia acima
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000SeekDir
; PARAMETROS.... ARFLAGS<SEEKDIR> = 0 --> sintonia abaixo
;                ARFLAGS<SEEKDIR> = 1 --> sintonia acima 
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000SeekOn
; PARAMETROS.... Nao ha
;
; Liga a sintonia.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000SeekOff
; PARAMETROS.... Nao ha
;
; Desliga a sintonia.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000GetFreq
; PARAMETROS.... Nao ha
;
; Le o valor da frequencia da estacao corrente em STATUS<READCHAN> (9 bits)
; devolvendo o resultado no registrador ARFREQ(2).
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000Write
; PARAMETROS.... W contem o registrador a ser gravado
;                FSR0 aponta um buffer de 2 bytes com ovalor a ser gravado
;
; Grava um registrador de 2 bytes no AR1000.
;
; ------------------------------------------------------------------------
; FUNCAO........ ar1000Read
; PARAMETROS.... W contem o registrador a ser lido
;                FSR0 aponta um buffer de leitura de 2 bytes
;
; Le um registrador de 2 bytes do AR1000.
;
; FUNCAO........ ar1000WaitSTC
; PARAMETROS.... Nao ha
;
; Aguarda STATUS<STC> = 1.
; Cuidado, pode ficar aguardando para sempre.
;
; ------------------------------------------------------------------------







; ------------------------------------------------------------------------
; COMO USAR ESTE SERVICO NUM PROGRAMA GENERICO:
; ------------------------------------------------------------------------
; Incluir os registradores abaixo no programa:
;I2C_CLOCK       equ   .20000000          ; 20MHZ - FOSC DO SISTEMA
;I2C_FREQ        equ     .100000          ; SLEWRATE
;AR1000PUT       equ        0x20          ; endereco write I2C do radio FM
;AR1000GET       equ        0x21          ; endereco read I2C do radio FM 
;ARFLAGS         RES          .1          ; Flags
;ARSEEKDIR       equ          .0          ; direcao da pesquisa 0->Up  1-->Down
;ARREGNUM        RES          .1
;ARWRK01         RES          .1
;ARNBYTES        RES          .1
;ARFREQ          RES          .2          ; frequencia
;ARREGS          RES         .36          ; buffer leitura/gravacao de registradores
;
; Incluir na memoria de programa o arquivo:
;        #include "..\..\Services\AR1000\ar1000.funcs.inc".
;
; 1 - Configurar o AR1000, chamando a funcao 'ar1000Config':
;
;           call    ar1000Config                  ; configuracao inicial do AR1000  
;           
;     Isto ja e' suficiente para o radio entrar em operacao.
;     O radio sera configurado com os valores defaults encontrados na lista
;     'DEFAULTS' ao final deste arquivo fonte. Cada par de valores nessa lista
;     sera carregado num registrador do AR1000. Modifique a lista 'DEFAULTS' de 
;     cordo com suas necessidades.
;     Os itens que seguem apenas mostram alguns exemplos de como o radio pode ser
;     controlado.
;
; 2 - Como sintonizar uma estacao:
;     Para sintonizar uma estacao usa-se a funcao 'ar1000SetFreq'.
;     A sintonizacao de uma estacao  feita gravando-se um valor no registrador
;     _R2 do AR1000 que devera ser calculado usando-se a seguinte formula:
;                     V = (10 * frequencia_real) - 690
;     Por exemplo, para sintonizar a radio Jovem Pan de Sao Paulo, cuja
;     frequencia real e' 100,9MHZ, fazemos:
;                     V = (10 * 100.9) - 690
;                     V = 1009 - 690
;                     V = 319
;     Carrega-se o registrador auxiliar 'ARFREQ' (de 2 bytes) com o valor 319 e chama-se
;     a funcao 'ar1000SetFreq':
;                     movlw  high(.319)
;                     movwf  ARFREQ
;                     movlw  low(.319)
;                     movwf  ARFREQ+.1
;                     call   ar1000SetFreq
;
;
; 3 - Para silenciar o radio, usar a funcao:
;                     call   ar1000Mute
; 
; 4 - Para escutar o som do radio, usar a funcao:
;                     call   ar1000Play
;
; 5 - Para aumentar o volume do som:
;                     movlw  nivel              ; onde 'nivel' e' um numero de 1 a 18
;                     call   ar1000SetVol
;
; 6 - Mudar automaticamente de estacao, para cima ou para baixo, a partir da estacao 
;     corrente usando a funcao 'ar1000Seek'.
;     Se o registrador W for carregado com o valor 0 (zero) a proxima estacao abaixo
;     sera sintonizada.
;     Se o registrador W for carregado com o valor 1 (um) a proxima estacao acima
;     sera sintonizada.
;     Exemplos:
;
;                     movlw  .0                 ; sintoniza proxima estacao abaixo da atual                
;                     call   ar1000Seek
;                     movlw  .1                 ; sintoniza proxima estacao acima da atual                
;                     call   ar1000Seek
;
;
;  7 - Como controlar o volume de som do radio:
;      O volume do radio e' controlado atraves dos registradores _R3 (D7-10) e _R14 (D12-15).
;      O volume pode ser controlado atraves de niveis que vao de 1 a 18. Cada nivel refere-se
;      a um valor contido na tabela 'VOLUMES' presente no final deste arquivo fonte.
;      Para modificar o volume use a funcao 'ar1000SetVol' passando no registrador W o nivel 
;      de volume desejado. Exemplo:
;
;                     movlw  .13              ; nivel de volume
;                     call   ar1000SetVol
;
;
; ------------------------------------------------------------------------



;-----------------------------------------------------------------------
; Definicao de registradores do radio FM AR1000               12/10/2019
;-----------------------------------------------------------------------
_R0           equ    .0
X0_EN         equ    .7
ENABLE        equ    .0
;
_R1           equ    .1
RDS_EN        equ    .13
RDS_INT_EN    equ    .6
STC_INT_EN    equ    .5
DEEMP         equ    .4
MONO          equ    .3
SMUTE         equ    .2
HMUTE         equ    .1
;
_R2           equ    .2
TUNE          equ    .9
CHAN8         equ    .8     
CHAN7         equ    .7 
CHAN6         equ    .6 
CHAN5         equ    .5 
CHAN4         equ    .4 
CHAN3         equ    .3 
CHAN2         equ    .2 
CHAN1         equ    .1 
CHAN0         equ    .0 
;
_R3           equ    .3
SEEKUP        equ    .15
SEEK          equ    .14
_SPACE        equ   .13
BAND1         equ    .12
BAND0         equ    .11
VOLUME3       equ    .10
VOLUME2       equ    .9
VOLUME1       equ    .8
VOLUME0       equ    .7
SEEKTH6       equ    .6
SEEKTH5       equ    .5
SEEKTH4       equ    .4
SEEKTH3       equ    .3
SEEKTH2       equ    .2
SEEKTH1       equ    .1
SEEKTH0       equ    .0
;
_R4           equ    .4
_R5           equ    .5
_R6           equ    .6
_R7           equ    .7
_R8           equ    .8
_R9           equ    .9
_R10          equ    .10
;
_R11          equ    .11
HILO_SIDE     equ    .15
HILOCTRL_B1   equ     .2
HILOCTRL_B2   equ     .0
;
_R12          equ    .12
;
_R13          equ    .13
GPIO31        equ    .5
GPIO30        equ    .4
GPIO21        equ    .3
GPIO20        equ    .2
GPIO11        equ    .1
GPIO10        equ    .0
;
_R14          equ    .14
VOLUME23      equ    .15
VOLUME22      equ    .14
VOLUME21      equ    .13
VOLUME20      equ    .12
;
_R15          equ   .15
RDS_STA_EN    equ    .5
RDS_MECC1     equ    .4
RDS_MEMM0     equ    .3
RDS_CTRL      equ    .0
;
_R16          equ   .16
_R17          equ    .17
RSSI          equ    .18
;
_STATUS       equ    .19
READCHAN8     equ    .15
READCHAN7     equ    .14
READCHAN6     equ    .13
READCHAN5     equ    .12
READCHAN4     equ    .11
READCHAN3     equ    .10
READCHAN2     equ     .9
READCHAN1     equ     .8
READCHAN0     equ     .7
RDSR          equ     .6
STC           equ     .5
SF            equ     .4
ST            equ     .3
;
RBS           equ    .20
RDS1          equ    .21
RDS2          equ    .22
RDS3          equ    .23
RDS4          equ    .24
RDS5          equ    .25
RDS6          equ    .26
DEVID         equ    .27
CHIPID        equ    .28


; ----------------------------------------------------------------------
; Configuracao inicial do AR1000                              16/10/2019
;
; Grava todos os registradores do AR1000 com os valores da tabela
; 'DEFAULTS' que se encontra ao final deste arquivo fonte.
;
; ----------------------------------------------------------------------
ar1000Config call    _AR1000DEF                  ; apontadores dos DEFAULTS 
             call    _AR1000MVC                  ; despreza _R0
             movlw   .1                          ; inicia no registrador _R1
             movwf   ARREGNUM                    ; carrega numero do registrador
             movlw   .17                         ; numero de registradores a gravar
             movwf   ARWRK01                     ; carrega contador
ar1000getb   movlw   .250
             call    delay_ms
             call    _AR1000MVC                  ; move um valor (2 bytes) para ARREGS
             movf    ARREGNUM, w                 ; pega o numero do registrador
             lfsr    FSR0, ARREGS                ; aponta valor a gravar
             call    ar1000Write                 ; grava 2 bytes no registrador
             incf    ARREGNUM, f                 ; proximo registtrador
             decfsz  ARWRK01, f                  ; acabou???
             goto    ar1000getb                  ; NAO --> continua
             call    _AR1000DEF                  ; recarrega os ponteiros de DEFAULTS
             call    _AR1000MVC                  ; obtem valor do registrador _R0
             movlw   .0                          ; registrador _R0
             lfsr    FSR0, ARREGS                ; aponta valor para _R0
             call    ar1000Write                 ; grava _R0
             call    ar1000TuneOn
             return

; ----------------------------------------------------------------------
; Subtrotina interna. Nao usar no programa.                   17/10/2019
; ----------------------------------------------------------------------
_AR1000DEF   movlw   UPPER(DEFAULTS)             ; endereco UPPER da origem na FLASH
             movwf   TBLPTRU
             movlw   HIGH(DEFAULTS)              ; endereco HIGH da origem na FLASH
             movwf   TBLPTRH
             movlw   LOW(DEFAULTS)               ; endereco LOW da origem na FLASH
             movwf   TBLPTRL
             return

; ----------------------------------------------------------------------
; Subtrotina interna. Nao usar no programa.                   17/10/2019
; ----------------------------------------------------------------------
_AR1000MVC   lfsr    FSR2, ARREGS
             tblrd   *+                       ; leitura da memoria FLASH
             movf    TABLAT, w                ; carrega W
             movwf   POSTINC2                 ; move para o destino
             tblrd   *+                       ; leitura da memoria FLASH
             movf    TABLAT, w                ; carrega W
             movwf   POSTINC2                 ; move para o destino
             return

; ----------------------------------------------------------------------
; Le todos os registradores do AR1000                         16/10/2019
;
; Coloca o resultado no buffer 'ARREGS' de 36 bytes.
;
; ----------------------------------------------------------------------
ar1000ReadAll
             movlw   _R0
             movwf   ARREGNUM
             movwf   .18
             movwf   ARWRK01
             lfsr    FSR0, ARREGS
ar1000Rd     movf    ARREGNUM, w
             call    ar1000Read
             incf    ARREGNUM, f
             decfsz  ARWRK01, f
             goto    ar1000Rd
             return

; ----------------------------------------------------------------------
; Insere a frequencia no R2                                   16/10/2019
;
; Entrada: O registrador ARFREQ(2) deve conter a frequencia real ja 
;          diminuida de 690.
; ----------------------------------------------------------------------
ar1000SetFreq
           call    ar1000TuneOff                

           movlw   _R2
           lfsr    FSR0, ARREGS
           call    ar1000Read

           movlw   0xFE
           andwf   ARREGS+.0, f
           movlw   0x00
           andwf   ARREGS+.1, f

           movf    ARFREQ, w
           iorwf   ARREGS+.0, f
           movf    ARFREQ+.1, w
           iorwf   ARREGS+.1, f

           movlw   _R2
           lfsr    FSR0, ARREGS
           call    ar1000Write

           movlw   .6                 ; aguarda 2 segundos
           movwf   ARNBYTES
ar100SetF1 movlw   .250
           call    delay_ms
           decfsz  ARNBYTES, f
           goto    ar100SetF1
          
           call    ar1000TuneOn
           return

; ----------------------------------------------------------------------
; Set TUNE BIT ON                                             16/10/2019
; ----------------------------------------------------------------------
ar1000TuneOn    
           movlw   _R2
           lfsr    FSR0, ARREGS                ; le _R2
           call    ar1000Read
           bsf     ARREGS+.0, 1                  ; TUNE ON
           movlw   _R2
           lfsr    FSR0, ARREGS                ; le _R2
           call    ar1000Write
           return

; ----------------------------------------------------------------------
; Set TUNE BIT OFF                                            16/10/2019
; ----------------------------------------------------------------------
ar1000TuneOff    
           movlw   _R2
           lfsr    FSR0, ARREGS                ; le _R2
           call    ar1000Read
           bcf     ARREGS+.0, 1                  ; TUNE OFF
           movlw   _R2
           lfsr    FSR0, ARREGS                ; le _R2
           call    ar1000Write
           return

; ----------------------------------------------------------------------
; FM MUTE (HMUTE)                                             16/10/2019
; ----------------------------------------------------------------------
ar1000Mute movlw   _R1
           lfsr    FSR0, ARREGS                ; le _R1
           call    ar1000Read
           bsf     ARREGS+.1, 1                ; MUTE
           movlw   _R1
           lfsr    FSR0, ARREGS                ; le _R1
           call    ar1000Write
           return

; ----------------------------------------------------------------------
; FM PLAY (HMUTE)                                             16/10/2019
; ----------------------------------------------------------------------
ar1000Play movlw   _R1
           lfsr    FSR0, ARREGS                ; le _R1
           call    ar1000Read
           bcf     ARREGS+.1, 1                ; PLAY
           movlw   _R1
           lfsr    FSR0, ARREGS                ; le _R1
           call    ar1000Write
           return

; -----------------------------------------------------------------------------------
; ar1000SetVol                                                             18/10/2019
;
; Modifica o volume do som do AR1000.
;
; Os valores do volume estao contidos na tabela VOLUMES e substituirao os valores dos
; registrador _R3 (D7-10) e _R14 (D12-15).
; 
; Entrada: W contem um nivel de volume de 0 a 18 que sera usado para indexar a
;          tabela 'VOLUMES' presente na memoria Flash.
;
; Saida  : W=0 --> volume invalido nao pode ser usado.
;          W=1 --> volume aceito e modificado.
;         
; -----------------------------------------------------------------------------------
ar1000SetVol   movwf   ARWRK01                   ; salva o indice 
               rlcf    ARWRK01, f                ; Indice X 2 
               bcf     ARWRK01, 0                ; limpa sugeira introduzida pelo Carry
               sublw   .18
               btfss   STATUS, C
               retlw   0                         ; ERRO: W<0 ---> volume > 18
                  
               ; Enderecamento da tabela 'VOLUMES' na memoria Flash
               movlw   UPPER(VOLUMES)
               movwf   TBLPTRU
               movlw   HIGH(VOLUMES)
               movwf   TBLPTRH
               movlw   LOW(VOLUMES)
               movwf   TBLPTRL

               ; Indexacao da tabela 'VOLUMES' com o indice recebido
               movf    ARWRK01, w
               addwf   TBLPTRL, f
               movf    STATUS, w
               andlw   0x01 
               addwf   TBLPTRH, f
               movf    STATUS, w
               andlw   0x01 
               addwf   TBLPTRU, f
       
               ; Leitura do valor do volume referente ao indice recebido
               tblrd   *+
               movff   TABLAT, ARWRK01      ; ARWRK01    = mmmm.llll  (pega o volume)
               movff   ARWRK01, ARREGS+.11  ; ARREGS<11> = mmmm.llll  (temporariamente)

               ; Modifica o LSB do volume em _R3
               movlw   _R3                  ; registrador _R3
               lfsr    FSR0, ARREGS         ; aponta buffer de leitura
               call    ar1000Read           ; le _R3 em ARREGS<0:1>
               movlw   0xF8
               andwf   ARREGS+.0, f         ; ARREGS<0> = xxxx.xVVV
               movlw   0x7F
               andwf   ARREGS+.1, f         ; ARREGS<1> = Vyyy.yyyy
                                            ; ARREGS<0:1> = xxxx.xVVV Vyyy.yyyy
               movlw   .7
               movwf   ARNBYTES
ar1000SetV01   rlcf    ARREGS+.11, f        ; ARREGS<10:11> = ????.???? mmmm.llll
               rlcf    ARREGS+.10, f
               decfsz  ARNBYTES, f
               goto    ar1000SetV01
               movlw   0x07                 ; ARREGS<10:11> = ?mmm.mlll l???.????
               andwf   ARREGS+.10, w        ; ARREGS<10> = 0000.0lll
               iorwf   ARREGS+.0, f         ; altera o buffer para gravar
               movlw   0x80
               andwf   ARREGS+.11, w        ; ARREGS<11> = l000.0000
               iorwf   ARREGS+.1, f         ; altera o buffer para gravar
               movlw   _R3                  ; registrador _R3
               lfsr    FSR0, ARREGS
               call    ar1000Write          ; grava _R3

               ; Modifica o MSB do volume em _R14
               movff   ARWRK01, ARREGS+.11  ; ARREGS<11> = mmmm.llll  (temporariamente)
               movlw   _R14                 ; registrador _R14
               lfsr    FSR0, ARREGS         ; aponta buffer de leitura
               call    ar1000Read           ; le _R14 em ARREGS<0:1>
               movlw   0x0F
               andwf   ARREGS+.0, f         ; ARREGS<0> = xxxx.xVVV
               movlw   0xFF
               andwf   ARREGS+.1, f         ; ARREGS<1> = Vyyy.yyyy
                                            ; ARREGS<0:1> = xxxx.xVVV Vyyy.yyy
               movlw   .8
               movwf   ARNBYTES
ar1000SetV02   rlcf    ARREGS+.11, f        ; ARREGS<10:11> = ????.???? mmmm.llll
               rlcf    ARREGS+.10, f
               decfsz  ARNBYTES, f
               goto    ar1000SetV02
                                            ; ARREGS<10:11> = mmmm.llll ????.????
               movlw   0xF0
               andwf   ARREGS+.10, w        ; ARREGS<10> = mmmm.0000
               iorwf   ARREGS+.0, f         ; altera o buffer para gravar
               movlw   0x00
               andwf   ARREGS+.11, w        ; ARREGS<11> = 0000.0000
               iorwf   ARREGS+.1, f         ; altera o buffer para gravar
               movlw   _R14                 ; registrador _R14
               lfsr    FSR0, ARREGS
               call    ar1000Write          ; grava _R14

               retlw   1                    ; OK: W=1
 
; ----------------------------------------------------------------------
; ar1000Seek - Pesquisa automatica da estacao                 22/10/2019
;
; Pesquisa a estacao automaticamente, subindo ou descendo a escala,
; de acordo com a direcao especificada no registrador W.
;
; Entrada: W=0 --> pesquisa abaixo da estacao corrente.
;          W=1 --> pesquisa acima  da estacao corrente.
;
; Saida  : Nao ha
; 
; ----------------------------------------------------------------------
ar1000Seek     bcf     ARFLAGS, ARSEEKDIR   ; assume pesquisa para baixo
               andlw   0x01                 ; deixa so o bit 0 (up/down)
               iorwf   ARFLAGS, f           ; ARFLAGS<ARSEEKDIR> - direcao
               call    ar1000SeekOff        ; desliga SEEK no _R3
               call    ar1000SeekDir        ; estabelece direcao da pesquisa

               ; Obtem a frequencia corrente de _STATUS<READCHAN>
               call    ar1000GetFreq        ; pega a frequencia atual (_STATUS)

               ; Grava a frequencia atual em _R2
               movlw   _R2                  ; le _R2
               lfsr    FSR0, ARREGS
               call    ar1000Read           ; ARREGS=xxxx.xxxc cccc.cccc
               movlw   0xFE        
               andwf   ARREGS+.0, f         ; ARREGS=xxxx.xxx0 cccc.cccc
               movlw   0x00        
               andwf   ARREGS+.1, f         ; ARREGS=xxxx.xxx0 0000.0000
               movf    ARFREQ+.0, w
               iorwf   ARREGS+.0, f         ; ARREGS=xxxx.xxxn 0000.0000
               movf    ARFREQ+.1, w
               iorwf   ARREGS+.1, f         ; ARREGS=xxxx.xxxn nnnn.nnnn
               movlw   _R2
               lfsr    FSR0, ARREGS
               call    ar1000Write

               ; Inicia a pesquisa
               call    ar1000SeekOn         ; religa o SEEK
               call    ar1000WaitSTC        ; aguarda completar a pesquisa
               call    ar1000Play           ; toca

               return  
      
; ----------------------------------------------------------------------
; ar1000SeekDir - Direcao do SEEK no _R3                      22/10/2019
;
; Especifica a direcao da pesquisa de estacoes de acordo com o bit
; ARFLAGS<ARSEEKDIR>.
;
; Entrada: ARFLAGS<ARSEEKDIR>=0 --> pesquisa abaixo da estacao corrente.
;          ARFLAGS<ARSEEKDIR>=1 --> pesquisa acima   da estacao corrente.
;
; Saida  : Registrador _R3 regravado.
;
; ----------------------------------------------------------------------
ar1000SeekDir  movlw   _R3
               lfsr    FSR0, ARREGS
               call    ar1000Read  
               bcf     ARREGS+.0, (SEEKUP - .8)  ; assume pesquisa abaixo
               btfsc   ARFLAGS, ARSEEKDIR        ; pesquisa acima???
               bsf     ARREGS+.0, (SEEKUP - .8)  ; SIM --> entao pesquisa acima 
               movlw   _R3
               lfsr    FSR0, ARREGS
               call    ar1000Write
               return

; ----------------------------------------------------------------------
; ar1000SeekOn - Liga o bit SEEK do _R3                       22/10/2019
; ----------------------------------------------------------------------
ar1000SeekOn   movlw   _R3
               lfsr    FSR0, ARREGS
               call    ar1000Read
               bsf     ARREGS+.0, (SEEK - .8)
               movlw   _R3
               lfsr    FSR0, ARREGS
               call    ar1000Write
               return

; ----------------------------------------------------------------------
; ar1000SeekOff - Desliga o bit SEEK do _R3                   22/10/2019
; ----------------------------------------------------------------------
ar1000SeekOff  movlw   _R3
               lfsr    FSR0, ARREGS
               call    ar1000Read
               bcf     ARREGS+.0, (SEEK - .8)
               movlw   _R3
               lfsr    FSR0, ARREGS
               call    ar1000Write
               return

; ----------------------------------------------------------------------
; ar1000GetFreq - Obtem frequencia da estacao corrente        22/10/2019
;
; Le o registrador _STATUS obtem a frequencia da estacao corrente e 
; devolve a mesma no registrador FREQ(2) alinhada a direita.
;
; Entrada: Nao ha
;
; Saida  : O registrador de 2 bytes ARFREQ contera a frequencia
;          (READCHAN bits) do registrador _STATUS. 
;
; ----------------------------------------------------------------------
ar1000GetFreq  movlw   _STATUS
               lfsr    FSR0, ARFREQ
               call    ar1000Read          ; FREQ=Mccc.cccc Lxxx.xxxx
               movlw   .7
               movwf   ARNBYTES
ar1000GetFq1   rrcf    ARFREQ+.0, f
               rrcf    ARFREQ+.1, f
               decfsz  ARNBYTES, f
               goto    ar1000GetFq1
               movlw   0x01                
               andwf   ARFREQ+.0, f        ; FREQ=0000.000M cccc.cccL
               return

; ----------------------------------------------------------------------
; Grava um registrador de 2 bytes do Radio FM AR1000          16/10/2019
;
; Entrada: W contem o numero do registrador
;          FSR0 aponta para o buffer de gravacao (2 bytes)
;
; Saida  : O registrador de 2 bytes contera o valor apontado por FSR0
;
; ----------------------------------------------------------------------
ar1000Write    movwf   ARREGNUM                       ; salva temporariamente 
               I2C_SHUTDOWN                          ; clrf   SSPCON1 
               I2C_CONFIG   I2C_CLOCK, I2C_FREQ, SLEWRATE
               call    i2c_start
               movlw   AR1000PUT                     ; endereco de escrita
               call    i2c_put                       ; envia
               call    i2c_waitACK                   ; espera MSSP
               movf    ARREGNUM, w                    ; numero do registrador
               call    i2c_put                       ; envia
               call    i2c_waitACK                   ; espera ACK
               movlw   .2                            ; numero de bytes a enviar
               movwf   ARNBYTES                      ; salva contador
_ar1000send    movf    POSTINC0, w                   ; pega um byte do buffer       
               call    i2c_put                       ; envia
               call    i2c_waitACK                   ; espera ACK
               decfsz  ARNBYTES                      ; acabou???
               goto    _ar1000send                   ; NAO --> envia o proximo
               call    i2c_stop                      ; Stop
               return

; ----------------------------------------------------------------------
; Le um registrador do Radio FM AR1000                        15/10/2019
;
; Entrada: W contem o numero do registrador
;          FSR0 aponta para o buffer de leitura (2 bytes)
;
; Saida  : O endereco apontado por FSR0 contera o valor lido (2 bytes)
;          do registrador especificado.
;
; ----------------------------------------------------------------------
ar1000Read movwf    ARREGNUM                       ; salva temporariamente 
           I2C_SHUTDOWN                           ; clrf   SSPCON1 
           I2C_CONFIG   I2C_CLOCK, I2C_FREQ, SLEWRATE
           call     i2c_start                     ; START
           movlw    AR1000PUT                     ; endereco de WRITE
           call     i2c_put                       ; envia
           call     i2c_waitACK                   ; espera ACK
           movf     ARREGNUM, w                    ; restaura o numero do registrador
           call     i2c_put                       ; envia
           call     i2c_waitACK                   ; espera ACK
           movlw    .2                            ; numero de bytes a serem lidos
           movwf    ARNBYTES                      ; carrega o contador
           call     i2c_restart                   ; RESTART
           movlw    AR1000GET                     ; endereco de leitura
           call     i2c_put                       ; envia
           call     i2c_waitACK                   ; espera ACK   
           movf     SSPBUF, w                     ; DUMMY READ
arloopler  call     i2c_get                       ; le um byte no dispositivo I2C
           movf     i2c_buff, w
           movwf    POSTINC0
           call     i2c_ack                       ; envia ACK para o dispositivo I2C
           decfsz   ARNBYTES, f                   ; atualiza o contador de bytes lidos
           goto     arloopler                     ; le o proximo byte
           call     i2c_get                       ; le ultimo byte
           movf     i2c_buff,w
           movwf    POSTINC0
           call     i2c_nack                      ; envia NACK
           call     i2c_stop                      ; stop para encerrar a leitura
           return

; ----------------------------------------------------------------------
; ar1000WaitSTC - Aguarda bit _STATUS<STC>==1                 22/10/2019
; ----------------------------------------------------------------------
ar1000WaitSTC  movlw   _STATUS              ; _STATUS
               lfsr    FSR0, ARREGS         ; buffer de leitura
               call    ar1000Read           ; le _STATUS
               btfsc   ARREGS+.1, STC       ; _STATUS<STC>==1???
               return                       ; SIM --> retorna
               movlw   .50                  ; tempo
               call    delay_ms             ; milisagundos 
               goto    ar1000WaitSTC        ; espera _STATUS<STC>


; ----------------------------------------------------------------------
; Array de valores para configuracao inicial do radio         06/10/2019
; ----------------------------------------------------------------------
DEFAULTS   DB      0xFF,0xFB  
           DB      0x5B,0x15  
           DB      0xD1,0x57     ; TUNE ON, 100.3MHZ
           DB      0xA0,0x10  
           DB      0x07,0x80  
           DB      0x28,0xAB  
           DB      0x64,0x00  
           DB      0x1E,0xE7  
           DB      0x71,0x41  
           DB      0x00,0x7D  
           DB      0x82,0xC6  
           DB      0x4E,0x55  
           DB      0x97,0x0C  
           DB      0xB8,0x45  
           DB      0xFC,0x2D  
           DB      0x80,0x97  
           DB      0x04,0xA1  
           DB      0xDF,0x61

; -----------------------------------------------------------------------------------
; Controle de Volume.
; MSB 4 bits R14<12:15> --> R14<VOLUME2>
; LSB 4 bits R3<7:10>   --> R3<VOLUME>
; -----------------------------------------------------------------------------------
VOLUMES    DB      0x0F           ; 0        
           DB      0xCF           ; 1     
           DB      0xDF           ; 2     
           DB      0xFF           ; 3     
           DB      0xCB           ; 4     
           DB      0xDB           ; 5     
           DB      0xFB           ; 6     
           DB      0xFA           ; 7     
           DB      0xF9           ; 8     
           DB      0xF8           ; 9     
           DB      0xF7           ; 10    
           DB      0xD6           ; 11    
           DB      0xE6           ; 12    
           DB      0xF6           ; 13    
           DB      0xE3           ; 14  
           DB      0xF3           ; 15    
           DB      0xF2           ; 16    
           DB      0xF1           ; 17    
           DB      0xF0           ; 18
    